<?php
/**
 * Class InstaQuotesApi
 *
 * @author Andrey O.
 * @copyright Copyright (c) 2016, InstaForex Group
 */

class InstaQuotesApi
{
    const VERSION = '1.0.0';

    private static $instance = array();

    /**
     * Экзепляр транспорта
     *
     * @var null
     */
    private $transport = NULL;

    /**
     * Текущая команда
     *
     * @var null
     */
    private $action = NULL;

    /**
     * Параметры запроса
     *
     * @var null
     */
    private $params = NULL;

    /**
     * Ответ на запрос
     *
     * @var null
     */
    private $response = NULL;

    /**
     * Создаем экземпляр API с соответствующим транспортом
     *
     * @param InstaQuotesApiTransportInterface $transport
     */
    private function __construct(InstaQuotesApiTransportInterface $transport)
    {
        $this->transport = $transport;
    }

    /**
     * Instance of API
     * @param InstaQuotesApiTransportInterface $transport
     * @return mixed
     */
    public static function getInstance(InstaQuotesApiTransportInterface $transport)
    {
        $id = serialize($transport);

        if(! isset($instance[$id]))
        {
            self::$instance[$id] = new self($transport);
        }

        return self::$instance[$id];
    }

    /**
     * Магический вызов метода в API
     *
     * @param $method
     * @param $params
     * @return InstaQuotesApi
     */
    public function __call($method, $params)
    {
        $this->action = $method;

        $mp = array();
        foreach($params as $p)
        {
            if(is_array($p))
            {
                $mp += $p;
            }
            else
            {
                $mp[] = $p;
            }
        }

        $this->params = $mp;

        return $this->send();
    }

    /**
     * Отправка запроса соответствующим транспортом
     *
     * @return $this
     */
    private function send()
    {
        $mergedParams = array(
            'action' => $this->action,
            'params' => array_merge($this->params, array('m' => 'json'))
        );

        $this->response = $this->transport->getConnection()
                                          ->params($mergedParams)
                                          ->send();

        $this->action = NULL;
        $this->params = NULL;

        return $this;
    }

    /**
     * Получение ассоциативного массива
     *
     * @param $array
     * @param $key
     * @return array
     */
    private static function getAssocArray($array, $key)
    {
        $ret = array();

        if(! empty($array) && is_array($array) && $key !== NULL)
        {
            foreach($array as $k => $v)
            {
                $nk = isset($v[$key]) ? $v[$key] : $k;
                $ret[$nk] = $v;
            }
        }
        else
        {
            $ret = $array;
        }

        return $ret;
    }

    /**
     * Получение чистого ответа
     *
     * @return string
     */
    public function getRaw()
    {
        return $this->response->result();
    }

    /**
     * Получение результата как массива
     *
     * @param null $key ключ как индекс
     * @return array
     */
    public function getAsArray($key = NULL)
    {
        $array = json_decode($this->getRaw(), true);
        return self::getAssocArray($array, $key);
    }

    /**
     * Получение результата как объекта
     *
     * @return mixed
     */
    public function getAsObject()
    {
        return json_decode($this->getRaw());
    }

    /**
     * Получение результата как массива (алиас к getAsArray)
     *
     * @param null $key ключ как индекс
     * @return array
     */
    public function get($key = NULL)
    {
        return $this->getAsArray($key);
    }
}