<?php
/**
 * Class InstaQuotesApiHttpTransport
 * Реализация HTTP транспорта посредством cURL
 *
 * @author Andrey O.
 * @copyright Copyright (c) 2016, InstaForex Group
 */

class InstaQuotesApiHttpTransport extends InstaQuotesApiTransportAbstract implements InstaQuotesApiTransportInterface
{
    private $connection = NULL;

    protected $config = array();

    private $out = '';

    /**
     * Опции для cURL по умолчанию
     * @var array
     */
    private $defaultCurlOptions = array(
        CURLOPT_HEADER         => FALSE,
        CURLOPT_RETURNTRANSFER => TRUE,
        CURLOPT_TIMEOUT        => 3,
        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_0
    );

    /**
     * Конфиг по умочанию
     * @var array
     */
    protected static $defaul_config = array(
        'host'   => 'quotes.instaforex.com',
        'port'   => 80
    );

    /**
     * Закрытие коннекта cURL при уничтожении класса
     */
    public function __destruct()
    {
        $this->close();
    }

    /**
     * Закрытие коннекта cURL
     */
    public function close()
    {
        if($this->connection)
        {
            curl_close($this->connection);
            $this->connection = NULL;
        }
    }

    /**
     * Возвращает экземпляр класса
     *
     * @param array $config
     * @return mixed
     */
    public static function getInstance(array $config = array())
    {
        $config = ! empty($config) ? $config : self::$defaul_config;

        return parent::getInstance(__CLASS__, $config);
    }

    /**
     * Инициализация cURL
     *
     * @return $this
     * @throws InstaQuotesApiException
     */
    public function getConnection()
    {
        if($this->connection === NULL)
        {
            $this->connection = curl_init();
            if($this->connection === FALSE)
            {
                throw new InstaQuotesApiException('cURL init false');
            }
        }

        return $this;
    }

    /**
     * Возращает готовый url для зарпоса на основе параметров
     *
     * @return string
     */
    private function buildUrl()
    {
        $host = isset($this->config['host']) ? $this->config['host'] : '';
        $port = isset($this->config['port']) ? $this->config['port'] : '';

        $url = $host . ':' . $port;
        $url = stripos($host, '://') === FALSE ? 'http://'.$url : '';

        if(isset($this->params['action']))
        {
            $url .= '/api/' . $this->params['action'];
        }

        if(isset($this->params['params']) && !empty($this->params['params']))
        {
            $url .= '?' . http_build_query($this->params['params']);
        }

        return $url;
    }

    /**
     * Отправка запроса посредством cURL
     *
     * @return $this
     * @throws InstaQuotesApiException
     */
    public function send()
    {
        if($this->connection)
        {

            $curlOpt = $this->defaultCurlOptions;
            $curlOpt[CURLOPT_URL] = $this->buildUrl();

            if(isset($this->config['curl']) && is_array($this->config['curl']))
            {
                $curlOpt += $this->config['curl'];
            }

            curl_setopt_array($this->connection, $curlOpt);

            $this->out = curl_exec($this->connection);

            if($this->out === FALSE)
            {
                throw new InstaQuotesApiException(sprintf('ERROR: %d - %s.', curl_errno($this->connection), curl_error($this->connection)));
            }

            $this->close();
        }

        return $this;
    }

    /**
     * Возвращает резутат запроса.
     * Очищает результат
     *
     * @return string
     */
    public function result()
    {
        $out = $this->out;
        $this->out = '';

        return $out;
    }
}