<?php
/**
 * Class InstaQuotesApiSocketTransport
 * Реализация Socket транспорта посредством socket_stream
 *
 * @author Andrey O.
 * @copyright Copyright (c) 2016, InstaForex Group
 */

class InstaQuotesApiSocketTransport extends InstaQuotesApiTransportAbstract implements InstaQuotesApiTransportInterface
{
    private $connection = NULL;

    protected $config = array();

    private $out = '';

    /**
     * Конфиг по умолчанию
     *
     * @var array
     */
    protected static $defaul_config = array(
        'host'   => 'quotes.instaforex.com',
        'port'   => 5000
    );

    public function __destruct()
    {
        $this->close();
    }

    /**
     * Закрытие сокета
     */
    public function close()
    {
        if($this->connection)
        {
            socket_close($this->connection);
            $this->connection = NULL;
        }
    }

    /**
     * Возвращает экземпляр класса
     *
     * @param array $config
     * @return mixed
     */
    public static function getInstance(array $config = array())
    {
        $config = ! empty($config) ? $config : self::$defaul_config;

        return parent::getInstance(__CLASS__, $config);
    }

    /**
     * Инициализация сокета
     *
     * @return $this
     * @throws InstaQuotesApiException
     */
    public function getConnection()
    {
        if($this->connection === NULL)
        {
            $this->connection = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
            if($this->connection === FALSE)
            {
                $error = socket_strerror(socket_last_error());
                throw new InstaQuotesApiException($error);
            }

            $result = @socket_connect($this->connection, $this->config['host'], $this->config['port']);
            if ($result === FALSE)
            {
                $error = socket_strerror(socket_last_error($this->connection));

                $this->connection = NULL;
                throw new InstaQuotesApiException($error);
            }

            socket_read($this->connection, 2048);
        }

        return $this;
    }

    /**
     * Отправка запроса посредством socket
     *
     * @return $this
     * @throws InstaQuotesApiException
     */
    public function send()
    {
        if($this->connection)
        {
            $params = json_encode($this->params) . PHP_EOL;

            socket_write($this->connection, $params, strlen($params));

            while (($out = socket_read($this->connection, 2048, PHP_NORMAL_READ)) != PHP_EOL)
            {
                $this->out .= $out;
            }
        }

        return $this;
    }

    /**
     * Возвращает резутат запроса.
     * Очищает результат
     *
     * @return string
     */
    public function result()
    {
        $out = $this->out;
        $this->out = '';

        return $out;
    }
}